
#include "configuration.h"
#include "declarations.h"


#if defined(USE_MQTT_BROKER) && (USE_MQTT_BROKER==1)

unsigned char CurrentStage = 0;       // текущий этап процесса
unsigned long TimeStage    = 0;       // Время работы на текущем этапе
unsigned int CntStop       = 0;       // Счетчик СТОПОВ

unsigned long SecPrevBuf   = 0;       // время (Seconds) предыдущего обновления данных дисплея
unsigned long SecPrevOper  = 0;       // время (Seconds) предыдущего обновления оперативных данных
unsigned long SecCmdDisp   = 0;       // время (Seconds) прихода команды, приводящей к изменению информации на дисплее

bool FlagRefreshOnline     = true;   // признак необходимости обновления данных в online режиме
bool NeedRefresh           = false;   // признак необходимости обновления показаний дисплея
bool extGenerator          = false;   // Генератор для проверки целостности линии связи автоматика-scada

String RecvBufSerial       = "";      // буфер для приема сообщений от ESP

char lcd_mqtt_buf1[26];               // первая строка экрана для отправки по MQTT
char lcd_mqtt_buf2[26];               // вторая строка экрана для отправки по MQTT

char buf_pmem[30];                    // буфер для считывания данных из PROGMEM


// ########################################################
// # Описание форматов для передачи на ESP
// ########################################################

// оперативные данные
PROGMEM const char fmt_lcd1[]           =  "lcd1=%s";                // первая строка дисплея
PROGMEM const char fmt_lcd2[]           =  "lcd2=%s";                // вторая строка дисплея
PROGMEM const char fmt_klpHLD[]         =  "klpHLD=%d";              // клапан холодильника (для дистилляции)
PROGMEM const char fmt_klpDEFL[]        =  "klpDEFL=%d";             // клапан для подачи воды в дефлегматор
PROGMEM const char fmt_klpGLV_HVS[]     =  "klpGLV_HVS=%d";          // клапан отбора головных и хвостовых фракций
PROGMEM const char fmt_klpSR[]          =  "klpSR=%d";               // клапан отбора ректификата
PROGMEM const char fmt_stage_0[]        =  "stage=0";                // этапы текущего процесса НЕ ЗАПУЩЕН
PROGMEM const char fmt_stage_1[]        =  "stage=1";                //ЗАПУСК             
PROGMEM const char fmt_stage_2[]        =  "stage=2";                //РАЗГОН
PROGMEM const char fmt_stage_3[]        =  "stage=3";                //НА СЕБЯ
PROGMEM const char fmt_stage_4[]        =  "stage=4";                //ОТБОР ГОЛОВ
PROGMEM const char fmt_stage_5[]        =  "stage=5";                //СТОП
PROGMEM const char fmt_stage_6[]        =  "stage=6";                //ОТБОР ТЕЛА
PROGMEM const char fmt_stage_7[]        =  "stage=7";                //ХВОСТЫ
PROGMEM const char fmt_stage_8[]        =  "stage=8";                //ОЖИДАНИЕ
PROGMEM const char fmt_stage_100[]      =  "stage=100";              //ЗАКОНЧЕН
PROGMEM const char fmt_stage_101[]      =  "stage=101";              //АВАРИЯ ТСА
PROGMEM const char fmt_stage_102[]      =  "stage=102";              //АВАРИЯ ДАВЛЕНИЕ
PROGMEM const char fmt_stage_def[]      =  "stage=404";              // НЕИЗВЕСТНО
PROGMEM const char fmt_t_kub[]          =  "t_kub=%d";                   // температура в кубе
PROGMEM const char fmt_t_col[]          =  "t_col=%d";                   // температура в колонне
PROGMEM const char fmt_t_tsa[]          =  "t_tsa=%d";                   // температура в ТСА
PROGMEM const char fmt_work_time[]      =  "work_time=%02u:%02u:%02u";   // время текущего процесса
PROGMEM const char fmt_ProcShimSR[]     =  "ProcShimSR=%d";              // % ШИМ отбора тела
PROGMEM const char fmt_tStabSR[]        =  "tStabSR=%d";                 // температура стабилизации колонны
PROGMEM const char fmt_CntStop[]        =  "CntStop=%u";                 // количество СТОПОВ
PROGMEM const char fmt_TimeStage[]      =  "TimeStage=%02u:%02u:%02u";   // время текущего этапа
PROGMEM const char fmt_UU_MPX5010[]     =  "UU_MPX5010=%d";              //// Текущее значение давления
PROGMEM const char fmt_extGenerator[]   =  "extGenerator=%d";            //// Генератор для проверки целостности линии связи автоматика-scada

// настройки
PROGMEM const char fmt_BeepKeyPress[]   =  "BeepKeyPress=%u";            // звук кнопок
PROGMEM const char fmt_BeepEndProc[]    =  "BeepEndProc=%u";             // звук окончания процесса
PROGMEM const char fmt_BeepStateProc[]  =  "BeepStateProc=%u";           // звук смены этапа
PROGMEM const char fmt_FlToUSART[]      =  "FlToUSART=%u";               // вывод в UART
PROGMEM const char fmt_tEndRectRazgon[] =  "tEndRectRazgon=%d";          // температура окончания разгона
PROGMEM const char fmt_tEndRectOtbGlv[] =  "tEndRectOtbGlv=%d";          // температура окончания отбора голов
PROGMEM const char fmt_tEndRectOtbSR[]  =  "tEndRectOtbSR=%d";           // температура окончания отбора тела
PROGMEM const char fmt_tEndRect[]       =  "tEndRect=%d";                // температура окончания ректификации
PROGMEM const char fmt_ShimGlv[]        =  "ShimGlv=%d";                 // ШИМ отбора голов
PROGMEM const char fmt_ProcShimGlv[]    =  "ProcShimGlv=%u";             // % ШИМ отбора голов
PROGMEM const char fmt_ShimSR[]         =  "ShimSR=%u";                  // ШИМ отбора тела
PROGMEM const char fmt_MinProcShimSR[]  =  "MinProcShimSR=%u";           // минимальный % ШИМ отбора тела
PROGMEM const char fmt_BegProcShimSR[]  =  "BegProcShimSR=%u";           // начальный % ШИМ отбора тела 
PROGMEM const char fmt_timeStabKolonna[]=  "timeStabKolonna=%d";         // Время стабилизации колонны TimeStabKolonna
PROGMEM const char fmt_tDeltaRect[]     =  "tDeltaRect=%d";              // дельта ректификации
PROGMEM const char fmt_ProvodSR[]       =  "ProvodSR=%d";                //// настройка окончания отбора голов
PROGMEM const char fmt_PowRect[]        =  "PowRect=%u";                 //// Мощность ректификации
PROGMEM const char fmt_PowerT[]         =  "Power=%u";                   //// Мощность тена
//PROGMEM const char fmt_PowGlvDistil[]   =  "PowGlvDistil=%u";            //// Мощность отбора голов при дистилляции
PROGMEM const char fmt_PowDistil[]      =  "PowDistil=%u";               //// Мощность дистилляции
PROGMEM const char fmt_Tem1P[]          =  "Tem1P=%d";                   //// температура окончания дистилляции 1
//PROGMEM const char fmt_Tem2P[]          =  "Tem2P=%d";                   //// температура окончания дистилляции 2
//PROGMEM const char fmt_Tem3P[]          =  "Tem3P=%d";                   //// температура окончания дистилляции 3
PROGMEM const char fmt_TDeflBegDistil[] =  "TDeflBegDistil=%d";          //// температура окончания разгона 
PROGMEM const char fmt_UPeregrev[]      =  "UPeregrev=%u";               //// какое U надо поддерживать для защиты клапанов от перегрева.
PROGMEM const char fmt_AvtonomHLD[]     =  "AvtonomHLD=%u";              //// Признак того, что используется автономная система охлаждения
PROGMEM const char fmt_iCorrectASC712[] =  "iCorrectASC712=%u";          //// Признак того, как надо использовать датчик asc712 
PROGMEM const char fmt_PAlarmMPX5010[]  =  "PAlarmMPX5010=%d";           //// Давление при котором надо выдавать сингал тревоги
PROGMEM const char fmt_ds1820popr0[]    =  "ds1820popr0=%d";             //// Поправка для датчика температуры 0
PROGMEM const char fmt_ds1820popr1[]    =  "ds1820popr1=%d";             //// Поправка для датчика температуры 1
PROGMEM const char fmt_ds1820popr2[]    =  "ds1820popr2=%d";             //// Поправка для датчика температуры 2
PROGMEM const char fmt_ds1820popr3[]    =  "ds1820popr3=%d";             //// Поправка для датчика температуры 3
PROGMEM const char fmt_PP_MPX5010[]     =  "PP_MPX5010=%d";              //// Поправка для датчика давления
//PROGMEM const char fmt_PU_MPX5010[]     =  "PU_MPX5010=%03d/%03u";     //// Поправка датчика давления и текушее давление
PROGMEM const char fmt_DDopInfo[]       =  "DDopInfo=%u";                //// Флаг отображения аварийной информации
PROGMEM const char fmt_CCHIM[]          =  "CCHIM=%d";                   //// Колличество точек при отборе по Т куба
PROGMEM const char fmt_tempKCHIM1[]     =  "tempKCHIM1=%d";              //// Температура в точке 1 при отборе по Т куба
PROGMEM const char fmt_CHIMTK1[]        =  "CHIMTK1=%u";                 //// % Шим в точке 1 при отборе по Т куба
PROGMEM const char fmt_tempKCHIM2[]     =  "tempKCHIM2=%d";              //// Температура в точке 2 при отборе по Т куба
PROGMEM const char fmt_CHIMTK2[]        =  "CHIMTK2=%u";                 //// % Шим в точке 2 при отборе по Т куба
PROGMEM const char fmt_tempKCHIM3[]     =  "tempKCHIM3=%d";              //// Температура в точке 3 при отборе по Т куба
PROGMEM const char fmt_CHIMTK3[]        =  "CHIMTK3=%u";                 //// % Шим в точке 3 при отборе по Т куба
PROGMEM const char fmt_tempKCHIM4[]     =  "tempKCHIM4=%d";              //// Температура в точке 4 при отборе по Т куба
PROGMEM const char fmt_CHIMTK4[]        =  "CHIMTK4=%u";                 //// % Шим в точке 4 при отборе по Т куба
PROGMEM const char fmt_tempKCHIM5[]     =  "tempKCHIM5=%d";              //// Температура в точке 5 при отборе по Т куба
PROGMEM const char fmt_CHIMTK5[]        =  "CHIMTK5=%u";                 //// % Шим в точке 5 при отборе по Т куба
PROGMEM const char fmt_tempKCHIM6[]     =  "tempKCHIM6=%d";              //// Температура в точке 6 при отборе по Т куба
PROGMEM const char fmt_CHIMTK6[]        =  "CHIMTK6=%u";                 //// % Шим в точке 6 при отборе по Т куба
PROGMEM const char fmt_IncChim[]        =  "IncChim=%d";                 //// Автоматическое увеличение % ШИМ отбора
PROGMEM const char fmt_DecChim[]        =  "DecChim=%d";                 //// Автоматическое уменьшение % ШИМ отбора
// ########################################################
// # Название изменяемых значений
// ########################################################
PROGMEM const char val_BeepKeyPress[]   =  "BeepKeyPress";            // звук кнопок
PROGMEM const char val_BeepEndProc[]    =  "BeepEndProc";             // звук окончания процесса
PROGMEM const char val_BeepStateProc[]  =  "BeepStateProc";           // звук смены этапа
PROGMEM const char val_FlToUSART[]      =  "FlToUSART";               // вывод в UART
PROGMEM const char val_tEndRectRazgon[] =  "tEndRectRazgon";          // температура окончания разгона
PROGMEM const char val_tEndRectOtbGlv[] =  "tEndRectOtbGlv";          // температура окончания отбора голов
PROGMEM const char val_tEndRectOtbSR[]  =  "tEndRectOtbSR";           // температура окончания отбора тела
PROGMEM const char val_tEndRect[]       =  "tEndRect";                // температура окончания ректификации
PROGMEM const char val_ShimGlv[]        =  "ShimGlv";                 // ШИМ отбора голов
PROGMEM const char val_ProcShimGlv[]    =  "ProcShimGlv";             // % ШИМ отбора голов
PROGMEM const char val_ShimSR[]         =  "ShimSR";                  // ШИМ отбора тела
PROGMEM const char val_ProcShimSR[]     =  "ProcShimSR";              // Текущий % ШИМ отбора тела
PROGMEM const char val_MinProcShimSR[]  =  "MinProcShimSR";           // минимальный % ШИМ отбора тела
PROGMEM const char val_BegProcShimSR[]  =  "BegProcShimSR";           // начальный % ШИМ отбора тела
PROGMEM const char val_timeStabKolonna[]=  "timeStabKolonna";         // Время стабилизации колонны TimeStabKolonna
PROGMEM const char val_tDeltaRect[]     =  "tDeltaRect";              // дельта ректификации
PROGMEM const char val_ProvodSR[]       =  "ProvodSR";                // настройка окончания отбора голов
PROGMEM const char val_online[]         =  "online";                  // настройка обновления дисплея
PROGMEM const char val_save[]           =  "save";                    // сохранение в EEPROM
PROGMEM const char val_refresh[]        =  "refresh";                 // считывание значений настроек
PROGMEM const char val_keystrokes[]     =  "keystrokes";              //// нажатие кнопки
//PROGMEM const char val_key_d[]          =  "key_d";                   // кнопка ВНИЗ
//PROGMEM const char val_key_u[]          =  "key_u";                   // кнопка ВВЕРХ
//PROGMEM const char val_key_l[]          =  "key_l";                   // кнопка ВЛЕВО
//PROGMEM const char val_key_r[]          =  "key_r";                   // кнопка ВПРАВО
//PROGMEM const char val_key_s[]          =  "key_s";                   // кнопка ВЫБРАТЬ
PROGMEM const char val_PowRect[]        =  "PowRect";                 // Мощность ректификации
PROGMEM const char val_PowerT[]         =  "PowerT";                  //// Мощность ТЭНа
//PROGMEM const char val_PowGlvDistil[]   =  "PowGlvDistil";            //// Мощность отбора голов при дистилляции
PROGMEM const char val_PowDistil[]      =  "PowDistil";               //// Мощность дистилляции
PROGMEM const char val_Tem1P[]          =  "Tem1P";                   //// температура дистилляции 1
//PROGMEM const char val_Tem2P[]          =  "Tem2P";                   //// температура дистилляции 2
//PROGMEM const char val_Tem3P[]          =  "Tem3P";                   //// температура дистилляции 3
PROGMEM const char val_TDeflBegDistil[] =  "TDeflBegDistil";          //// температура окончания разгона
PROGMEM const char val_UPeregrev[]      =  "UPeregrev";               //// какое U надо поддерживать для защиты клапанов от перегрева.
PROGMEM const char val_AvtonomHLD[]     =  "AvtonomHLD";              //// Признак того, что используется автономная система охлаждения
PROGMEM const char val_iCorrectASC712[] =  "iCorrectASC712";          //// Признак того, как надо использовать датчик asc712 
PROGMEM const char val_PAlarmMPX5010[]  =  "PAlarmMPX5010";           //// Давление при котором надо выдавать сингал тревоги.
PROGMEM const char val_ds1820popr0[]    =  "ds1820popr0";             //// Поправка для датчика температуры 0
PROGMEM const char val_ds1820popr1[]    =  "ds1820popr1";             //// Поправка для датчика температуры 1
PROGMEM const char val_ds1820popr2[]    =  "ds1820popr2";             //// Поправка для датчика температуры 2
PROGMEM const char val_ds1820popr3[]    =  "ds1820popr3";             //// Поправка для датчика температуры 3
PROGMEM const char val_PP_MPX5010[]     =  "PP_MPX5010";              //// Поправка для датчика давления
PROGMEM const char val_DDopInfo[]       =  "DDopInfo";                //// Флаг отображения аварийной информации
PROGMEM const char val_CCHIM[]          =  "CCHIM";                   //// Количество точек при отборе по Т куба
PROGMEM const char val_tempKCHIM1[]     =  "tempKCHIM1";              //// Температура в точке 1 при отборе по Т куба
PROGMEM const char val_CHIMTK1[]        =  "CHIMTK1";                 //// % Шим в точке 1 при отборе по Т куба
PROGMEM const char val_tempKCHIM2[]     =  "tempKCHIM2";              //// Температура в точке 2 при отборе по Т куба
PROGMEM const char val_CHIMTK2[]        =  "CHIMTK2";                 //// % Шим в точке 2 при отборе по Т куба
PROGMEM const char val_tempKCHIM3[]     =  "tempKCHIM3";              //// Температура в точке 3 при отборе по Т куба
PROGMEM const char val_CHIMTK3[]        =  "CHIMTK3";                 //// % Шим в точке 3 при отборе по Т куба
PROGMEM const char val_tempKCHIM4[]     =  "tempKCHIM4";              //// Температура в точке 4 при отборе по Т куба
PROGMEM const char val_CHIMTK4[]        =  "CHIMTK4";                 //// % Шим в точке 4 при отборе по Т куба
PROGMEM const char val_tempKCHIM5[]     =  "tempKCHIM5";              //// Температура в точке 5 при отборе по Т куба
PROGMEM const char val_CHIMTK5[]        =  "CHIMTK5";                 //// % Шим в точке 5 при отборе по Т куба
PROGMEM const char val_tempKCHIM6[]     =  "tempKCHIM6";              //// Температура в точке 6 при отборе по Т куба
PROGMEM const char val_CHIMTK6[]        =  "CHIMTK6";                 //// % Шим в точке 6 при отборе по Т куба
PROGMEM const char val_IncChim[]        =  "IncChim";                 //// Автоматическое увеличение % ШИМ отбора
PROGMEM const char val_DecChim[]        =  "DecChim";                 //// Автоматическое уменьшение % ШИМ отбора

// Считывание данных из PROGMEM
  char* GePMEM(
  char* ret_fmt,  // указатель на буфер для возвращаемого значения
  char* pmem_str  // указатель на строку (по имени переменной) из PROGMEM
)
{

int i;
int len;
  
  len = strlen_P(pmem_str);
  
  for (i=0; i<len; i++)
    ret_fmt[i] =  pgm_read_byte(pmem_str + i);
  
  ret_fmt[len] = '\0';

return ret_fmt;
}

void printSerial3(char *s) {
   // Ждем освобождения буфера отправки
  while (Serial3.availableForWrite() < strlen(s)+1)
    yield();

  Serial3.write(s, strlen(s)+1);
  //// Отправка топиков в Serial monitor, для отладки раскомментировать
  //Serial.println(s);  
     
}


// Отправка оперативных данных
void SendOperData() {

  char buf[30];


  // этот блок требуется для возможности расчета времени работы на текущем этапе и количества СТОПОВ
  // если произошла смена этапа
  if (CurrentStage != StateMachine) {

    // сохраним новый этап
    CurrentStage = StateMachine;
      
    //  если перешли в СТОП
    if (StateMachine == 5) {
            
      // увеличим счетчик стопов
      CntStop++;
    }

    // сохраним время перехода на новый этап
    TimeStage = Seconds;
        
  }
  
  // если наступило время отправки и еще не отправляли в эту секунду и разрешено обновление в online режиме
  // ИЛИ требуется обновление по нажатию кнопки и дисплей успел обновиться (задержка 1 сек)
  if ( (Seconds % 2 == 0 && Seconds != SecPrevBuf && FlagRefreshOnline) || (NeedRefresh && Seconds > SecCmdDisp+1) ) {
   
    // Вывод LCD экрана
    printSerial3(lcd_mqtt_buf1);
    printSerial3(lcd_mqtt_buf2);
    
     ////Флаг отображения аварийной информации
    sprintf(buf, GePMEM(buf_pmem,fmt_DDopInfo), DispDopInfo);
    printSerial3(buf);    
     //// состояние клапана отбора голов
    sprintf(buf, GePMEM(buf_pmem,fmt_klpGLV_HVS), KlState[KLP_GLV_HVS]);
    printSerial3(buf);
    //// состояние клапана отбора спирта
    sprintf(buf, GePMEM(buf_pmem,fmt_klpSR), KlState[KLP_SR]);
    printSerial3(buf);    
         
    // сохраним время отправки
    SecPrevBuf = Seconds;

    // сбросим признак обновления
    NeedRefresh = false;

  }
  
  // если наступило время отправки и еще не отправляли в эту секунду
  if (Seconds % 30 == 0 && Seconds != SecPrevOper) {
    
    //// Вывод LCD экрана
    printSerial3(lcd_mqtt_buf1);
    printSerial3(lcd_mqtt_buf2);
    
    // этап процесса
    switch (StateMachine) {
      case 0:   sprintf(buf, GePMEM(buf_pmem,fmt_stage_0));    break;
      case 1:   sprintf(buf, GePMEM(buf_pmem,fmt_stage_1));    break;
      case 2:   sprintf(buf, GePMEM(buf_pmem,fmt_stage_2));    break;
      case 3:   sprintf(buf, GePMEM(buf_pmem,fmt_stage_3));    break;
      case 4:   sprintf(buf, GePMEM(buf_pmem,fmt_stage_4));    break;
      case 5:   sprintf(buf, GePMEM(buf_pmem,fmt_stage_5));    break;
      case 6:   sprintf(buf, GePMEM(buf_pmem,fmt_stage_6));    break;
      case 7:   sprintf(buf, GePMEM(buf_pmem,fmt_stage_7));    break;
      case 8:   sprintf(buf, GePMEM(buf_pmem,fmt_stage_8));    break;
      case 100: sprintf(buf, GePMEM(buf_pmem,fmt_stage_100));  break;
      case 101: sprintf(buf, GePMEM(buf_pmem,fmt_stage_101));  break;
      case 102: sprintf(buf, GePMEM(buf_pmem,fmt_stage_102));  break;
      default:  sprintf(buf, GePMEM(buf_pmem,fmt_stage_def));  break;
    }
    printSerial3(buf);
    
    ////Флаг отображения аварийной информации
    sprintf(buf, GePMEM(buf_pmem,fmt_DDopInfo), DispDopInfo);
    printSerial3(buf); 
       
    //// температура в кубе
    sprintf(buf, GePMEM(buf_pmem,fmt_t_kub), temps[TEMP_KUB]);
    printSerial3(buf);
               
    // температура в колонне
    sprintf(buf, GePMEM(buf_pmem,fmt_t_col), temps[TEMP_RK20]);
    printSerial3(buf);
                
    // температура в TSA
    sprintf(buf, GePMEM(buf_pmem,fmt_t_tsa), temps[TEMP_TSA]);
    printSerial3(buf);
            
    // время с момента включения системы/начала процесса
    sprintf(buf, GePMEM(buf_pmem,fmt_work_time), hour, minute, second);
    printSerial3(buf);

    // текущий % отбора спирта
    sprintf(buf, GePMEM(buf_pmem,fmt_ProcShimSR), KLP_GLV_HVS != KLP_SR
                                                  // условие с двумя клапанами отбора
                                                  ? (KlOpen[KLP_GLV_HVS] != 0 ? KlOpen[KLP_GLV_HVS]/(timeChimRectOtbGlv/100) : KlOpen[KLP_SR]/(timeChimRectOtbSR/100))
                                                  // условие с одним клапаном отбора
                                                  : (StateMachine !=6 ? KlOpen[KLP_GLV_HVS]/(timeChimRectOtbGlv/100) : KlOpen[KLP_SR]/(timeChimRectOtbSR/100)) );
    printSerial3(buf);

    // температура стабилизации колонны
    sprintf(buf, GePMEM(buf_pmem,fmt_tStabSR), tStabSR);
    printSerial3(buf);

    // количество СТОПОВ
    sprintf(buf, GePMEM(buf_pmem,fmt_CntStop), CntStop);
    printSerial3(buf);

    // время текущего этапа
    sprintf(buf, GePMEM(buf_pmem,fmt_TimeStage), (int)((Seconds-TimeStage)/3600), (int)((Seconds-TimeStage)%3600)/60, (int)((Seconds-TimeStage)%3600)%60);
    printSerial3(buf);

    //// Текущее значение давления   
    sprintf(buf, GePMEM(buf_pmem,fmt_UU_MPX5010), U_MPX5010);   
    printSerial3(buf);   

    //// состояние клапана холодильника
    sprintf(buf, GePMEM(buf_pmem,fmt_klpHLD), KlState[KLP_HLD]);
    printSerial3(buf);
    //// состояние клапана для подачи воды в дефлегматор
    sprintf(buf, GePMEM(buf_pmem,fmt_klpDEFL), KlState[KLP_DEFL]);
    printSerial3(buf);  
    //// Генератор для проверки целостности линии связи автоматика-scada
       sprintf(buf, GePMEM(buf_pmem,fmt_extGenerator), extGenerator=!extGenerator);
    printSerial3(buf);           
      
    // сохраним время отправки
    SecPrevOper = Seconds;

  }  
}


// Отправка статических данных (настроек)
void SendStaticData() {

  char buf[30];
  
      //### отправляем данные ###
    
      //1 звук при нажатии клавиш
      sprintf(buf, GePMEM(buf_pmem,fmt_BeepKeyPress), BeepKeyPress);
      printSerial3(buf);
      
      //2 звук при окончании процесса
      sprintf(buf, GePMEM(buf_pmem,fmt_BeepEndProc), BeepEndProcess);
      printSerial3(buf);
      
      //3 звук при переходе между этапами процесса
      sprintf(buf, GePMEM(buf_pmem,fmt_BeepStateProc), BeepStateProcess);
      printSerial3(buf);
              
      //4 флаг вывода в USART
      sprintf(buf, GePMEM(buf_pmem,fmt_FlToUSART), FlToUSART);
      printSerial3(buf);

      //5 температура окончания режима разгона
      sprintf(buf, GePMEM(buf_pmem,fmt_tEndRectRazgon), tEndRectRazgon);
      printSerial3(buf);

      //6 температура окончания отбора голов
      sprintf(buf, GePMEM(buf_pmem,fmt_tEndRectOtbGlv), tEndRectOtbGlv);
      printSerial3(buf);

      //7 температура окончания отбора спирта
      sprintf(buf, GePMEM(buf_pmem,fmt_tEndRectOtbSR), tEndRectOtbSR);
      printSerial3(buf);

      //8 температура окончания ректификации
      sprintf(buf, GePMEM(buf_pmem,fmt_tEndRect), tEndRect);
      printSerial3(buf);

      //9 дельта ректификации
      sprintf(buf, GePMEM(buf_pmem,fmt_tDeltaRect), tDeltaRect);
      printSerial3(buf);      
      
      //10 ШИМ отбора голов
      sprintf(buf, GePMEM(buf_pmem,fmt_ShimGlv), timeChimRectOtbGlv/100);
      printSerial3(buf);

      //11 % ШИМ отбора голов
      sprintf(buf, GePMEM(buf_pmem,fmt_ProcShimGlv), ProcChimOtbGlv);
      printSerial3(buf);

      //12 ШИМ отбора спирта
      sprintf(buf, GePMEM(buf_pmem,fmt_ShimSR), timeChimRectOtbSR/100);
      printSerial3(buf);
      
      ////13 Время стабилизации колонны TimeStabKolonna
      sprintf(buf, GePMEM(buf_pmem,fmt_timeStabKolonna), TimeStabKolonna);
      printSerial3(buf);
      
      //14 минимальный % отбора спирта
      sprintf(buf, GePMEM(buf_pmem,fmt_MinProcShimSR), minProcChimOtbSR);
      printSerial3(buf);

      //15 начальный % отбора спирта
      sprintf(buf, GePMEM(buf_pmem,fmt_BegProcShimSR), begProcChimOtbSR);
      printSerial3(buf);

      ////16 режим отбора голов
      sprintf(buf, GePMEM(buf_pmem,fmt_ProvodSR), UrovenProvodimostSR);
      printSerial3(buf);
      
      ////17 Мощность ректификации
      sprintf(buf, GePMEM(buf_pmem,fmt_PowRect), PowerRect);
      printSerial3(buf);  
      ////18 мощность установленного ТЭНа
      sprintf(buf, GePMEM(buf_pmem,fmt_PowerT), Power);
      printSerial3(buf);    
      //// настройка мощности отбора голов при дистилляции
      //sprintf(buf, GePMEM(buf_pmem,fmt_PowGlvDistil), PowerGlvDistil);
      //printSerial3(buf);  
      ////19 настройка мощности дистилляции
      sprintf(buf, GePMEM(buf_pmem,fmt_PowDistil), PowerDistil);
      printSerial3(buf);    
      ////20 температура окончания дистилляции 1
      sprintf(buf, GePMEM(buf_pmem,fmt_Tem1P), Temp1P);
      printSerial3(buf);   
      //// температура окончания дистилляции 2
      //sprintf(buf, GePMEM(buf_pmem,fmt_Tem2P), Temp2P);
      //printSerial3(buf);    
      //// температура окончания дистилляции 3
      //sprintf(buf, GePMEM(buf_pmem,fmt_Tem3P), Temp3P);
      //printSerial3(buf);    
      ////21 температура окончания режима разгона
      sprintf(buf, GePMEM(buf_pmem,fmt_TDeflBegDistil), TempDeflBegDistil);
      printSerial3(buf);
      ////22 какое U надо поддерживать для защиты клапанов от перегрева.
      sprintf(buf, GePMEM(buf_pmem,fmt_UPeregrev), NaprPeregrev);   
      printSerial3(buf);
      ////23 Признак того, что используется автономная система охлаждения
      sprintf(buf, GePMEM(buf_pmem,fmt_AvtonomHLD), FlAvtonom);   
      printSerial3(buf);
      ////24 Признак того, как надо использовать датчик asc712 
      sprintf(buf, GePMEM(buf_pmem,fmt_iCorrectASC712), CorrectASC712);   
      printSerial3(buf);
      ////25 Давление при котором надо выдавать сингал тревоги.
      sprintf(buf, GePMEM(buf_pmem,fmt_PAlarmMPX5010), AlarmMPX5010);   
      printSerial3(buf);  
      ////26 Поправка для датчика давления
       sprintf(buf, GePMEM(buf_pmem,fmt_PP_MPX5010), P_MPX5010/10);   
       printSerial3(buf);
       
       ////27 Поправка для датчика температуры 0
       sprintf(buf, GePMEM(buf_pmem,fmt_ds1820popr0), ds1820_popr[0]);   
       printSerial3(buf);
       ////28 Поправка для датчика температуры 1
       sprintf(buf, GePMEM(buf_pmem,fmt_ds1820popr1), ds1820_popr[1]);   
       printSerial3(buf);
       ////29 Поправка для датчика температуры 2
       sprintf(buf, GePMEM(buf_pmem,fmt_ds1820popr2), ds1820_popr[2]);   
       printSerial3(buf);
       ////30 Поправка для датчика температуры 3
       sprintf(buf, GePMEM(buf_pmem,fmt_ds1820popr3), ds1820_popr[3]);   
       printSerial3(buf);  
       ////31 Колличество точек при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_CCHIM), CntCHIM);
       printSerial3(buf);
       ////32 Температура в точке при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_tempKCHIM1), tempK[1]);
       printSerial3(buf);      
       ////33 % Шим в точеке при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_CHIMTK1), CHIM[1]);
       printSerial3(buf);
       //// Температура в точке при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_tempKCHIM2), tempK[2]);
       printSerial3(buf);      
       ////34 % Шим в точеке при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_CHIMTK2), CHIM[2]);
       printSerial3(buf);
       ////35 Температура в точке при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_tempKCHIM3), tempK[3]);
       printSerial3(buf);      
       ////36 % Шим в точеке при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_CHIMTK3), CHIM[3]);
       printSerial3(buf);
       ////37 Температура в точке при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_tempKCHIM4), tempK[4]);
       printSerial3(buf);      
       ////38 % Шим в точеке при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_CHIMTK4), CHIM[4]);
       printSerial3(buf);
       ////39 Температура в точке при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_tempKCHIM5), tempK[5]);
       printSerial3(buf);      
       ////40 % Шим в точеке при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_CHIMTK5), CHIM[5]);
       printSerial3(buf);
       ////41 Температура в точке при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_tempKCHIM6), tempK[6]);
       printSerial3(buf);      
       ///42/ % Шим в точеке при отборе по Т куба
       sprintf(buf, GePMEM(buf_pmem,fmt_CHIMTK6), CHIM[6]);
       printSerial3(buf);
       ////43 Автоматическое увеличение % ШИМ отбора
       sprintf(buf, GePMEM(buf_pmem,fmt_IncChim), IncrementCHIM);
       printSerial3(buf);
       ////44 Автоматическое уменьшение % ШИМ отбора
       sprintf(buf, GePMEM(buf_pmem,fmt_DecChim), DecrementCHIM);
       printSerial3(buf);

}

  
// Получение данных от ESP
void RecvSerial3() {

  char ch;
  String pub_topic="";
  String val="";
  int pos_delim;
  int ret;
  unsigned int u_ret;
  
    while (Serial3.available()) {

      ch = (char)Serial3.read();

      if (ch == '\r') continue;

      if (ch == '\n') {

        // Проверим наличие разделителя в строке
        pos_delim = RecvBufSerial.indexOf("=");

        // Если разделитель присутствует
        if (pos_delim != -1 ) {
          
          // Выделим свойство и значение
          pub_topic = RecvBufSerial.substring(0,pos_delim);
          val = RecvBufSerial.substring(pos_delim+1);
                  
          // звук кнопок
          if ( pub_topic == GePMEM(buf_pmem,val_BeepKeyPress) )
            BeepKeyPress = (char)val.toInt();
          
          // звук окончания процесса
          else if ( pub_topic == GePMEM(buf_pmem,val_BeepEndProc) )
            BeepEndProcess = (char)val.toInt();
          
          // звук смены этапа
          else if ( pub_topic == GePMEM(buf_pmem,val_BeepStateProc) )
            BeepStateProcess = (char)val.toInt();    
          
          // вывод в UART
          else if ( pub_topic == GePMEM(buf_pmem,val_FlToUSART) )
            FlToUSART = (char)val.toInt();
          
          //// нажатие кнопки дисплея
          //else if ( pub_topic == GePMEM(buf_pmem,val_key_u) || pub_topic == GePMEM(buf_pmem,val_key_d) ||
          //          pub_topic == GePMEM(buf_pmem,val_key_l) || pub_topic == GePMEM(buf_pmem,val_key_r) ||
          //          pub_topic == GePMEM(buf_pmem,val_key_s) ) {
          //cmdWiFi = (char)val.charAt(0);
          else if ( pub_topic == GePMEM(buf_pmem,val_keystrokes) ) {   //// изменено для уменьшения количества переменных
            cmdWiFi = (char)val.charAt(0);
            // сохранить время получения команды
            SecCmdDisp = Seconds;
            // установить признак необходимости обновления показаний дисплея
            NeedRefresh = true;
          }
          
          // настройка обновления дисплея
          else if ( pub_topic == GePMEM(buf_pmem,val_online) ) {
            if (val.toInt() == 0) FlagRefreshOnline = false;
            else FlagRefreshOnline = true;
          }
          
          // сохранение в EEPROM
          else if ( pub_topic == GePMEM(buf_pmem,val_save) )
            writeEEPROM();
          
          // считывание значений настроек
          else if ( pub_topic == GePMEM(buf_pmem,val_refresh) )  
            SendStaticData();

          // температура окончания разгона
          else if ( pub_topic == GePMEM(buf_pmem,val_tEndRectRazgon) )  {
            ret = val.toInt();
            if ( ret != 0 and abs(ret) <= 999 )
              tEndRectRazgon = ret;
          }  

          // температура окончания отбора голов
          else if ( pub_topic == GePMEM(buf_pmem,val_tEndRectOtbGlv) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 and ret <= 999 )
              tEndRectOtbGlv = ret;
          }

          // температура окончания отбора тела
          else if ( pub_topic == GePMEM(buf_pmem,val_tEndRectOtbSR) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 and ret <= 999 )
              tEndRectOtbSR = ret;
          }

          // температура окончания ректификации
          else if ( pub_topic == GePMEM(buf_pmem,val_tEndRect) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 and ret <= 1250 )
              tEndRect = ret;
          }

          // ШИМ отбора голов
          else if ( pub_topic == GePMEM(buf_pmem,val_ShimGlv) )  {
            u_ret = val.toInt()*100;
            if ( u_ret != 0 and u_ret >= 0 )
              timeChimRectOtbGlv = u_ret;
          }

          // % ШИМ отбора голов
          else if ( pub_topic == GePMEM(buf_pmem,val_ProcShimGlv) )  {
            ret = val.toInt();
            if ( ret != 0 and abs(ret) <= 100 )
              ProcChimOtbGlv = ret;
          }

          // ШИМ отбора тела
          else if ( pub_topic == GePMEM(buf_pmem,val_ShimSR) )  {
            ret = val.toInt()*100;
            if ( ret != 0 and ret >= 0)
              timeChimRectOtbSR = ret;
          }
            //// текущий % ШИМ отбора тела
          else if ( pub_topic == GePMEM(buf_pmem,val_ProcShimSR) )  {
            ret = val.toInt();
            if ( ret != 0 and abs(ret) <= 100 )
              ProcChimSR = ret;
          }

          // минимальный % ШИМ отбора тела
          else if ( pub_topic == GePMEM(buf_pmem,val_MinProcShimSR) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 )
              minProcChimOtbSR = ret;
          }

          // начальный % ШИМ отбора тела
          else if ( pub_topic == GePMEM(buf_pmem,val_BegProcShimSR) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 )
              begProcChimOtbSR = ret;
          }
          
          //// Время стабилизации колонны TimeStabKolonna    
          else if ( pub_topic == GePMEM(buf_pmem,val_timeStabKolonna ) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 )
            TimeStabKolonna  = ret;
          }
          
          // дельта ректификации
          else if ( pub_topic == GePMEM(buf_pmem,val_tDeltaRect) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 )
              tDeltaRect = ret;
          }

          //// настройка окончания отбора голов
          else if ( pub_topic == GePMEM(buf_pmem,val_ProvodSR) )  {
            ret = val.toInt();
            if ( ret != 0 )
              UrovenProvodimostSR = ret;
          }
           // настройка мощности ректификации
          else if ( pub_topic == GePMEM(buf_pmem,val_PowRect) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 and ret <= Power )
              PowerRect = ret;
          }
         //// мощность установленного ТЭНа
          else if ( pub_topic == GePMEM(buf_pmem,val_PowerT) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 )
              Power = ret;
          }
          
          //// настройка мощности отбора голов при дистилляции
          //else if ( pub_topic == GePMEM(buf_pmem,val_PowGlvDistil) )  {
            //ret = val.toInt();
            //if ( ret != 0 and ret >= 0 and ret <= Power )
            //  PowerGlvDistil = ret;
          //}
          //// настройка мощности дистилляции
          else if ( pub_topic == GePMEM(buf_pmem,val_PowDistil) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 and ret <= Power )
              PowerDistil = ret;
          }
          //// температура окончания дистилляции 1
          else if ( pub_topic == GePMEM(buf_pmem,val_Tem1P) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 and ret <= 1250 )
              Temp1P = ret;
          }
          //// температура окончания дистилляции 2
          //else if ( pub_topic == GePMEM(buf_pmem,val_Tem2P) )  {
          //  ret = val.toInt();
          //  if ( ret != 0 and ret >= 0 and ret <= 1250 )
          //    Temp2P = ret;
          //}
          //// температура окончания дистилляции 3
          //else if ( pub_topic == GePMEM(buf_pmem,val_Tem3P) )  {
          //  ret = val.toInt();
          //  if ( ret != 0 and ret >= 0 and ret <= 1250 )
          //    Temp3P = ret;
          //}
          //// температура окончания разгона
          else if ( pub_topic == GePMEM(buf_pmem,val_TDeflBegDistil) )  {
            ret = val.toInt();
            if ( ret != 0 and abs(ret) <= 1250 )
              TempDeflBegDistil = ret;
          }  
          //// какое U надо поддерживать для защиты клапанов от перегрева.
          else if ( pub_topic == GePMEM(buf_pmem,val_UPeregrev) )  {
            ret = val.toInt();
            if ( ret != 0 and ret >= 0 and ret <= 300 )
              NaprPeregrev = ret;
          }  
          //// Признак того, что используется автономная система охлаждения
          else if ( pub_topic == GePMEM(buf_pmem,val_AvtonomHLD) )  {
            ret = (char)val.toInt();
            FlAvtonom = ret;        
          }  
          //// Признак того, как надо использовать датчик asc712 
          else if ( pub_topic == GePMEM(buf_pmem,val_iCorrectASC712) )  {
             ret = val.toInt();
            if ( ret >= 0 and ret <= 2 );
             CorrectASC712 = ret;
          }  
          //// Давление при котором надо выдавать сингал тревоги.
          else if ( pub_topic == GePMEM(buf_pmem,val_PAlarmMPX5010) )  {
             ret = val.toInt(); 
             if ( ret >= 0 );
             AlarmMPX5010 = ret;
          }  
                          
          //// Поправка для датчика давления
          else if ( pub_topic == GePMEM(buf_pmem,val_PP_MPX5010) )  {
           ret = val.toInt();
          
           P_MPX5010 = ret;
          }
          //// Поправка для датчика температуры 0
           else if ( pub_topic == GePMEM(buf_pmem,val_ds1820popr0) )  {
           ret = val.toInt();
         
           ds1820_popr[0] = ret;
           }
          //// Поправка для датчика температуры 1
           else if ( pub_topic == GePMEM(buf_pmem,val_ds1820popr1) )  {
           ret = val.toInt();
          
           ds1820_popr[1] = ret;           
           }
          //// Поправка для датчика температуры 2
           else if ( pub_topic == GePMEM(buf_pmem,val_ds1820popr2) )  {
           ret = val.toInt();
          
           ds1820_popr[2] = ret;           
           }  
          //// Поправка для датчика температуры 3
           else if ( pub_topic == GePMEM(buf_pmem,val_ds1820popr3) )  {
           ret = val.toInt();
         
           ds1820_popr[3] = ret;           
           }
          //// Количество точек при отборе по Т куба
           else if ( pub_topic == GePMEM(buf_pmem,val_CCHIM) )  {
           ret = val.toInt();
           if ( ret >= 0 and ret <= 15 );
           CntCHIM = ret;           
           }
           //// Температура в точке 1 при отборе по Т куба    
           else if ( pub_topic == GePMEM(buf_pmem,val_tempKCHIM1) )  {
           ret = val.toInt();
           if ( ret != 0 and ret >= 0 and ret <= 1250 )
           tempK[1] = ret;           
           }
            //// % Шим в точке 1 при отборе по Т куба
           else if ( pub_topic == GePMEM(buf_pmem,val_CHIMTK1) )  {
           ret = val.toInt();
           if ( ret != 0 and ret >= 0 )
           CHIM[1] = ret;          
           }
           //// Температура в точке 2 при отборе по Т куба    
           else if ( pub_topic == GePMEM(buf_pmem,val_tempKCHIM2) )  {
           ret = val.toInt();
           if ( ret != 0 and ret >= 0 and ret <= 1250 )
           tempK[2] = ret;           
           }
            //// % Шим в точке 2 при отборе по Т куба
           else if ( pub_topic == GePMEM(buf_pmem,val_CHIMTK2) )  {
           ret = val.toInt();
           if ( ret != 0 and ret >= 0 )
           CHIM[2] = ret;          
           }
           //// Температура в точке 3 при отборе по Т куба    
           else if ( pub_topic == GePMEM(buf_pmem,val_tempKCHIM3) )  {
           ret = val.toInt();
           if ( ret != 0 and ret >= 0 and ret <= 1250 )
           tempK[3] = ret;           
           }
            //// % Шим в точке 3 при отборе по Т куба
           else if ( pub_topic == GePMEM(buf_pmem,val_CHIMTK3) )  {
           ret = val.toInt();
           if ( ret != 0 and ret >= 0 )
           CHIM[3] = ret;          
           }
           //// Температура в точке 4 при отборе по Т куба    
           else if ( pub_topic == GePMEM(buf_pmem,val_tempKCHIM4) )  {
           ret = val.toInt();
           if ( ret != 0 and ret >= 0 and ret <= 1250 )
           tempK[4] = ret;           
           }
            //// % Шим в точке 4 при отборе по Т куба
           else if ( pub_topic == GePMEM(buf_pmem,val_CHIMTK4) )  {
           ret = val.toInt();
           if ( ret != 0 and ret >= 0 )
           CHIM[4] = ret;          
           }
            //// Температура в точке 5 при отборе по Т куба    
           else if ( pub_topic == GePMEM(buf_pmem,val_tempKCHIM5) )  {
           ret = val.toInt();
           if ( ret != 0 and ret >= 0 and ret <= 1250 )
           tempK[5] = ret;           
           }
            //// % Шим в точке 5 при отборе по Т куба
           else if ( pub_topic == GePMEM(buf_pmem,val_CHIMTK5) )  {
           ret = val.toInt();
           if ( ret != 0 and ret >= 0 )
           CHIM[5] = ret;          
           } 
             //// Температура в точке 6 при отборе по Т куба    
           else if ( pub_topic == GePMEM(buf_pmem,val_tempKCHIM6) )  {
           ret = val.toInt();
           if ( ret != 0 and ret >= 0 and ret <= 1250 )
           tempK[6] = ret;           
           }
            //// % Шим в точке 6 при отборе по Т куба
           else if ( pub_topic == GePMEM(buf_pmem,val_CHIMTK6) )  {
           ret = val.toInt();
           if ( ret != 0 and ret >= 0 )
           CHIM[6] = ret;          
           }  
            //// Автоматическое увеличение % ШИМ отбора
           else if ( pub_topic == GePMEM(buf_pmem,val_IncChim) )  {
           ret = val.toInt();
          
           IncrementCHIM = ret;          
           } 
            //// Автоматическое уменьшение % ШИМ отбора
           else if ( pub_topic == GePMEM(buf_pmem,val_DecChim) )  {
           ret = val.toInt();
           
           DecrementCHIM = ret;          
           } 
                             
                                    
        }    

        RecvBufSerial = "";  
        break;
      }
        
      RecvBufSerial += ch;
    }
    
}


#endif // #
//#################################################################################################################
